from pynput import keyboard
class InputHandler(object):
    """This handles input and output"""
    keys = []
    listener = None
    onTaps = []
    def __init__(self):
        self.keys = []
        self.listener = None
        self.onTaps = []
    def startListening(self):
        self.listener = keyboard.Listener(
            on_press=self.handleDown,
            on_release=self.handleUp
        )
        self.listener.start()
    def handleDown(self, key):
        if len(str(key)) == 3 and str(key)[0] == "'":
            say = str(key)[1]
        else:
            say = str(key)
        for func in self.onTaps:
            func(say)
        if len(str(key)) == 3 and str(key)[0] == "'":
            if str(key)[1] not in self.keys: self.keys.append(str(key)[1])
        else:
            if str(key) not in self.keys: self.keys.append(str(key))
    def handleUp(self, key):
        if len(str(key)) == 3 and str(key)[0] == "'":
            if str(key)[1] in self.keys: self.keys.remove(str(key)[1])
        else:
            if str(key) in self.keys: self.keys.remove(str(key))
    def getKey(self, key):
        if key in self.keys:
            return True
        else:
            return False
    def addListener(self, func):
        self.onTaps.append(func)



import sys
import os
import time
import cl_games.input as input
import cl_games.sprites as sprites
class Game:
    """This is the class for the game
        It takes no parameters
        This class is abstract, do not instance it, extend it.
        Just use Game().run() for running it
    """
    sprites = None
    renderLayers = []
    running = True
    title = ""
    sizeX = 40
    sizeY = 20
    wait = 0.02
    clock = 0
    lastFrame = 0
    hud = []
    blank = " "
    input = None
    frameTime = 0
    def __init__(self):
        self.sprites = sprites.SpriteGroup()
        self.renderLayers = []
        self.running = True
        self.title = ""
        self.sizeX = 40
        self.sizeY = 20
        self.wait = 0.02
        self.clock = 0
        self.lastFrame = 0
        self.hud = []
        self.blank = " "
        self.input = None
        self.frameTime = 0
    def update(self):
        pass
    def renderFrame(self):
        yAxis = []
        yAxis.append("_" * (self.sizeX + 2))
        yAxis.append("|" + (" " * self.sizeX) + "|\r" + ("|" + (" " * (int(self.sizeX / 2) - int(len(self.title) / 2)) + self.title)))
        for y in range(self.sizeY):
            xAxis = ""
            xAxis += "|"
            for x in range(self.sizeX):
                pixel = self.blank
                if self.renderLayers == []:
                    for sprite in self.sprites:
                        if y in range(int(sprite.posY),int(sprite.posY) + sprite.lenY) and x in range(int(sprite.posX), int(sprite.posX) + sprite.lenX):
                            if sprite.alive == True:
                                disY = y - int(sprite.posY)
                                disX = x - int(sprite.posX)
                                component = sprite.image[disY][disX]
                                pixel = component
                else:
                    for sgroup in self.renderLayers:
                        for sprite in sgroup:
                            if y in range(int(sprite.posY),int(sprite.posY) + sprite.lenY) and x in range(int(sprite.posX), int(sprite.posX) + sprite.lenX):
                                if sprite.alive == True:
                                    disY = y - int(sprite.posY)
                                    disX = x - int(sprite.posX)
                                    component = sprite.image[disY][disX]
                                    pixel = component

                xAxis += pixel
            xAxis += "|"
            yAxis.append(xAxis)
        for index, line in enumerate(self.hud):
            yAxis[index + 2] += '\r' +"|" + line
        yAxis.append("|" + ("_"*self.sizeX) + "|")
        frame = ""
        for line in yAxis:
            frame += "\n" + line
        return frame
    def onFrame(self):
        if self.renderLayers == []:
            for sprite in self.sprites:
                if  sprite.alive: sprite.update()
        else:
            for layer in self.renderLayers:
                for sprite in layer:
                    if  sprite.alive: sprite.update()
        self.update()
        frame = self.renderFrame()
        os.system('cls' if os.name == 'nt' else 'clear')
        sys.stdout.write(frame)
        sys.stdout.flush()
    def run(self):
        self.running = True
        if self.input is not None:
            self.input.startListening()
        lastFrame = time.time()
        while self.running:
            self.clock = time.time()
            if (self.clock - self.lastFrame) > self.wait:
                self.frameTime = self.clock - self.lastFrame
                self.lastFrame = self.clock
                self.onFrame()
    def addSprite(self, sprite, number=None):
        self.sprites.add(sprite)



class Sprite:
    """Sprite Class"""
    groups = []
    posX = 0
    posY = 0
    lenX = 0
    lenY = 0
    image = []
    alive = True
    def __init__(self, game):
        self.game = game
        self.groups = []
        self.posX = 0
        self.posY = 0
        self.lenX = 0
        self.lenY = 0
        self.image = []
    def update(self):
        pass
    def setLengthToImage(self):
        self.lenY = len(self.image)
        self.lenX = len(self.image[0])
    def kill(self):
        self.alive = False
        for sg in self.groups:
            sg.takeout(self)
class SpriteGroup:
    """docstring for SpriteGroup."""
    sprites = []
    def __init__(self):
        super(SpriteGroup, self).__init__()
        self.sprites = []
    def add(self, sprite):
        self.sprites.append(sprite)
        sprite.groups.append(self)
    def __iter__(self):
        return iter(self.sprites)
    def takeout(self, sprite):
        if sprite in self.sprites:
            self.sprites.remove(sprite)
        if self in sprite.groups:
            sprite.groups.remove(self)
